<?php
/**
 * Plugin Adm_redacteurs
 * Version  : 6.8 (2016/05/01)
 * Compatibility : Guppy v5.0+
 * Author   : jérôme CROUX (jchouix)
 * Web site : http://lebrikabrak.info/
 * E-mail   : lebrikabrak@free.fr
 * Licence  : GNU Lesser General Public License
 */

defined('ADMR_DIRNAME') OR exit('No direct script access allowed');

require_once ADMR_HELPERS_PATH.'/add_record_helper.php';

abstract class Add_doc_helper extends Add_record_helper
{
    /*
    |--------------------------------------------------------------------------
    | Properties
    |--------------------------------------------------------------------------
    */
    protected $CI;                       // object global
    protected $_lang = array();          // Données de langue
    protected $_config = array();        // Données de configuration  
    protected $_record_id = 0;           // Identifiant de l'enregistrement
    protected $_record_data = array();   // Tableau contenant les données de l'enregistrement
    protected $_form_params = array();   // Tableau contenant les données de l'enregistrement postées dans le formulaire
 	protected $_form_errors = array();	 // Tableau contenant les messages d'erreur dans le formulaire
    protected $_form_validation = FALSE; // Validation du formulaire (TRUE ou FALSE)

    protected $_doc_infos = array('type' => '', 'name' => ''); // Infos concernant le document (type, name)
    protected $_writer_rights = array(); // Droits du rédacteur
    
    /*
    |--------------------------------------------------------------------------
    | Constructor
    |--------------------------------------------------------------------------
    */    
    function __construct($doc_type = '', $writer_rights = array())
    {
        $this -> _doc_infos = $this -> _set_doc_infos($doc_type); // à définir en premier (important)
        $this -> _writer_rights = $this -> _set_writer_rights($writer_rights);

        parent :: __construct();

        $this -> CI -> load_model($this -> _doc_infos['name'].'_model'); // Model du type de document                      
        $this -> CI -> load_model('writer_docs_model', array('id' => $this -> _writer_rights['id']));        

        if (! ADMR_IS_ADMIN && ! $this -> _is_doc_allowed($this -> _record_id)) {
            Url_helper :: redirect('display_error&type=disallowed');
        }
    }

    /*
    |--------------------------------------------------------------------------
    | Methods
    |--------------------------------------------------------------------------
    */    

    /**
	 * Index
     * 
	 * @access public
     * @return void
	 */	
    public function index()
    {
        parent :: index();
    }

	// --------------------------------------------------------------------

    /*
     * Définit et renvoie le titre de la page 
     *
     * @param array Données à faire passer à la vue
     * @return void
     */        
    public function set_title_page()
    {
        return ucfirst($this -> _lang['add_'.$this -> _doc_infos['name']]);
    }

	// --------------------------------------------------------------------

	/**
	 * définit et renvoie les ajouts (js et css) dans l'entête de la page
     * 
     * @access public
	 * @return string Chaine html contenant les ajouts js et css
	 */	
    public function set_header_addons()
    {
        $js = '';
        $js .= '<script type="text/javascript" src="'.ADMR_JS_PATH.'/placeholder/jquery.placeholder.min.js"></script>'."\n";
        $js .= '<script type="text/javascript" src="'.ADMR_JS_PATH.'/datetimepicker/jquery.datetimepicker.min.js"></script>'."\n";
        $js .= '<script type="text/javascript" src="'.ADMR_JS_PATH.'/form-validator/jquery.form-validator.min.js"></script>'."\n";
        $js .= '<script type="text/javascript">$.formUtils.loadModules("module_add_doc", "'.ADMR_JS_PATH.'/form-validator/my_modules/");</script>'."\n";
//        $js .= '<script type="text/javascript" src="'.ADMR_JS_PATH.'/waiting_process.js"></script>'."\n";        
        $js .= '<script type="text/javascript" src="'.ADMR_JS_PATH.'/add_doc.js"></script>'."\n";
       
        $css = '';
        $css .= '<link rel="stylesheet" type="text/css" href="'.ADMR_CSS_PATH.'/datetimepicker/jquery.datetimepicker.css" />'."\n";
        $css .= '<link rel="stylesheet" type="text/css" href="'.ADMR_CSS_PATH.'/form_validator.css" />'."\n";
//        $css .= '<link type="text/css" rel="stylesheet" href="'.ADMR_CSS_PATH.'/waiting_process.css" />';        
        $css .= '<link rel="stylesheet" type="text/css" href="'.ADMR_CSS_PATH.'/add_doc.css" />'."\n";

        return $js.$css;
    }
    
	// --------------------------------------------------------------------

	/**
	 * définit et renvoie la description de la page ainsi que les recommendations
     * 
	 * @return string infos de la page
	 */	
    public function set_infos_page_content()
    {		
        return '';
    }

	// --------------------------------------------------------------------

    /*
     * Définit le contenu du menu qui est affiché dans le bandeau en haut de la page
     *
     * @return array Tableau contenant les items du menu (vide par défaut)
     */
    public function set_menu_plugin_items()
    {
        if (ADMR_IS_ADMIN) {
            $menu_plugin_items = Plugin_menu_helper :: set_admin_menu_items($this -> _lang, $this -> _config);    
        } else {
            $this -> CI -> load_helper('plugin_menu_for_'.$this -> _doc_infos['name'].'_helper');
            
            $menu_plugin_items = $this -> CI -> {'plugin_menu_for_'.$this -> _doc_infos['name'].'_helper'} -> set_public_items($this -> _doc_infos['name'], $this -> _lang, $this -> _config, $this -> _writer_rights);        
        }

        return $menu_plugin_items;
    }


	// --------------------------------------------------------------------

	/**
	 * définit les propriétés du document (type et nom)
     * 
     * @access protected
     * @param string Type du document
	 * @return array Tableau contenant les informations (type, nom) du document
	 */	
    protected function _set_doc_infos($doc_type = '')
    {
        $this -> _doc_infos = Model :: get_doc_infos_by_type($doc_type);

        if (empty($this -> _doc_infos['type']) || empty($this -> _doc_infos['name'])) {
            die('ERROR : vous devez définir le type et le nom du document');    
        }        
        
        return $this -> _doc_infos;        
    }

	// --------------------------------------------------------------------

	/**
	 * définit les droits du rédacteur
     * 
     * @access protected
     * @param array Tabelau définissant les droits du rédacteur (vide par défaut) 
	 * @return array Tableau contenant les droits du rédacteur
	 */	
    protected function _set_writer_rights($writer_rights = array())
    {
        settype($writer_rights, 'array');
        
        return $this -> _writer_rights = $writer_rights;        
    }

	// --------------------------------------------------------------------

	/**
	 * construit et retourne le contenu de l'attribut action du formulaire
     * 
     * @access protected
     * @param mixed Options du menu select
	 * @return void
	 */	
    protected function _build_form_action_content()
    {
        return Url_helper :: build_uri(array(
            'id'      => $this -> _record_id,
            'idpg'    => Var_helper :: import('idpg', '', 1, array('trim'), array(array('Var_helper', 'is_digit'))),
            'doctype' => trim(Var_helper :: import('doctype',  '', NULL, array('trim'), array(array('Var_helper', 'is_alpha_num_dash'))), '-'),
            'sort'    => Var_helper :: import('sort', '', $this -> _config['sort_option_by_default'], array('trim'), array(array('Var_helper', 'is_alpha_num_dash'))),
        ));
    }

	// --------------------------------------------------------------------

    /**
	 * Ajoute des données supplémentaires pour la vue sous la forme array(key => value)
     * 
	 * @access protected
     * @return array
	 */	
    protected function _add_more_data_for_add_record_view()
    {
        return array(
            'writer_rights' => $this -> _writer_rights,
            'cats' => $this -> _get_cats(),
            'doc_type' => $this -> _doc_infos,
            'private_groups' => $this -> _get_private_members_groups(),
        );
    }

	// --------------------------------------------------------------------

    /**
	 * Définit et retourne le layout de la vue
     * 
	 * @access protected
     * @return string nom du fichier qui définit le layout
	 */	
    protected function _get_layout_for_add_record_view()
    {
        return 'add_'.$this -> _doc_infos['name'].'_layout';    
    }

	// --------------------------------------------------------------------

    /**
	 * Ajoute ou modifie un enregistrement
     * 
	 * @access protected
     * @return boolean
	 */	
    protected function _add_record()
    {
        return parent :: _add_record();
    }

	// --------------------------------------------------------------------

    /**
	 * Crée un nouvel enregistrement dans la base de donnée
     * 
	 * @access protected
     * @param array Tableau contenant les données de l'enregistrement
     * @return integer Identifiant de l'enregistrement créé
	 */
    protected function _insert_record($record)
    {
        $record['id'] = $this -> CI -> {$this -> _doc_infos['name'].'_model'} -> insert($record); // Guppy
        $this -> CI -> writer_docs_model -> insert($record); // Plugin                
    
        return $record['id'];    
    }
    
	// --------------------------------------------------------------------

    /**
	 * Met à jours les données de l'enregistrement dans la base de donnée
     * 
	 * @access protected
     * @param array Tableau contenant les données de l'enregistrement
     * @return void
	 */
    protected function _update_record($record)
    {
        $this -> CI -> {$this -> _doc_infos['name'].'_model'} -> update($record);  // Guppy
        $this -> CI -> writer_docs_model -> update_mode_by_id($record['id'], $mode = 2); // Plugin
    
        return TRUE;    
    }
    
	// --------------------------------------------------------------------

    /**
	 * Ajoute d'autres actions après l'ajout ou la mise à jour d'un document
     * 
	 * @access protected
     * @return void
	 */
    protected function _add_more_action_after_adding_record()
    {
        $params =& $this -> _form_params;
        
        if (!ADMR_IS_ADMIN && !empty($this -> _writer_rights[$this -> _doc_infos['name'].'_send_email'])) {
            $this -> _send_email_to_inform_admin_about_record($params); // Envoie d'un email à l'administrateur   
        }        
    }
    
	// --------------------------------------------------------------------

    /**
	 * Envoie un email pour informer l'admin au sujet de l'ajout ou de la modification d'un document
     * 
	 * @access protected
     * @param array Tableau contenant les données de l'enregistrement
     * @return void
	 */
    protected function _send_email_to_inform_admin_about_record($record)
    {
        $this -> CI -> load_helper('email_helper');
        
        Email_helper :: inform_admin_of_writer_doc_modification(
            $writer_infos = array('name' => $this -> _writer_rights['name'], 'id' => $this -> _writer_rights['id']),
            $doc_infos = array('type' => $this -> _doc_infos['type'], 'name' => $this -> _doc_infos['name'], 'id' => $record['id']),
            $this -> _lang
        );
         
         return TRUE;        
    }

	// --------------------------------------------------------------------
    
    /**
	 * Récupère et renvoie les données du document
     * 
	 * @access protected
     * @param integer Id de l'enregistrement
     * @return array Données enregistrées du document
	 */    
    protected function _get_record_data($record_id)
    {
        $this -> CI -> load_model($this -> _doc_infos['name'].'_model'); // Model du type de document                      

        return  $this -> CI -> {$this -> _doc_infos['name'].'_model'} -> read($record_id);       
    }

	// --------------------------------------------------------------------
    
    /**
	 * Définit les valeurs des paramètres du formulaire
     * 
	 * @access protected
     * @param integer Id de l'enregistrement
     * @return array
	 */    
    protected function _set_form_params()
    {
        return parent :: _set_form_params();
    }

	// --------------------------------------------------------------------
    
    /**
	 * Récupère et renvoie les paramètres postés dans le formulaire
     * 
	 * @access protected
     * @return array Tableau contenant les paramètres postés dans le formulaire
	 */    
    protected function _get_form_params_posted()
    {
        $params = parent :: _get_form_params_posted();
        
        if (!empty($params)) {
            $params = $this -> _apply_writer_rights_for_cats_posted($params);       // Prise en compte ou non des catégories           
            $params = $this -> _apply_writer_rights_for_pubdates_posted($params);   // Prise en compte ou non des dates de publication         
            $params = $this -> _apply_writer_rights_for_status_posted($params);     // Prise en compte ou non du status           
        }
      
        return $params; // Paramètres postés dans le formulaire       
    }

	// --------------------------------------------------------------------
    
    /**
	 * Vérifie et corrige si nécessaire le status du document postés en fonction des autorisations du rédacteur
     * 
	 * @access private
     * @param array Données postées
     * @return array Données vérifiées et postées
	 */    
    private function _apply_writer_rights_for_status_posted($params)
    {
        if (empty($this -> _writer_rights[$this -> _doc_infos['name'].'_act'])) {
            $params['status'] = $this -> _config['doc_status_by_default']; // Status du document par défaut          
        }
        
        return  $params;       
    }

	// --------------------------------------------------------------------
    
    /**
	 * Vérifie et corrige si nécessaire les dates de publication postées en fonction des autorisations du rédacteur
     * 
	 * @access private
     * @param array Données postées
     * @return array Données vérifiées et postées
	 */    
    private function _apply_writer_rights_for_pubdates_posted($params)
    {
        if (empty($this -> _writer_rights[$this -> _doc_infos['name'].'_publication_manage'])) {            
            foreach(array('pubdate0', 'pubdate1', 'pubhour0', 'pubhour1', 'pubminute0', 'pubminute1') as $pubdate) {
                if (isset($params[$pubdate])) {
                    unset($params[$pubdate]);    
                }
            }
        }
        
        return  $params;       
    }

	// --------------------------------------------------------------------
    
    /**
	 * Vérifie et corrige si nécessaire les catégories postées en fonction des autorisations du rédacteur
     * 
	 * @access private
     * @param array Données postées
     * @return array Données vérifiées et postées
	 */    
    private function _apply_writer_rights_for_cats_posted($params)
    {
        $writer_rights = $this -> _writer_rights;
        $doc_name = $this -> _doc_infos['name'];

        foreach($this -> _config['lang'] as $key => $lng) {
            if (!empty($lng)) {
                $cat = 'cat'.($key + 1);
                
                if (empty($writer_rights[$doc_name.'_cats_manage'])) { // Pas d'autorisation de gérer les catégories
                    if (isset($params[$cat])) {
                        unset($params[$cat]);    
                    }
                }
                elseif ($writer_rights[$doc_name.'_cats_manage'] == 1) { // Gestion des catégories autorisées
                    if (empty($params[$cat]) && !empty($params['cat_extra'.($key + 1)])) { // Ajout de nouvelles catégories si nécessaire (cat_extra)
                        $params[$cat] = $params['cat_extra'.($key + 1)];     
                    }
                }                
                elseif ($writer_rights[$doc_name.'_cats_manage'] == 2) { // Catégories restreintes autorisées uniquement
                    if (isset($params[$cat]) && !in_array($params[$cat], $writer_rights[$doc_name.'_cats_list_allowed'][$key])) {
                        unset($params[$cat]);    
                    }
                }                
            }
        }
        
        return  $params;       
    }

	// --------------------------------------------------------------------
    
    /**
	 * Récupère et renvoie les paramètres avec leur valeur par défaut dans le formulaire
     * 
	 * @access protected
     * @return array Tableau contenant les paramètres postés dans le formulaire
	 */    
    protected function _get_form_params_by_default()
    {
        return  parent :: _get_form_params_by_default();      
    }

	// --------------------------------------------------------------------

    /**
	 * définit et renvoie les règles de validation des paramètres du formulaire
     * 
	 * @access protected
     * @param array Tableau contenant les paramètres à vérifer lors de la validation
     * @return array Tableau contenant les règles de validation des paramètres ainsi que les messages d'erreur associés 
     * sous la forme suivante : array(param1 => array(
     *                                               array('rules' => '', 'msg' => ''), // error 1
     *                                               array('rules' => '', 'msg' => ''), // error 2                                       
     *                                         ),
     *                                param2 => array(
     *                                               array('rules' => '', 'msg' => ''), // error 1
     *                                               array('rules' => '', 'msg' => ''), // error 2                                       
     *                                         ),
     *                               ...
     *                          )
	 */	
    protected function _get_form_params_rules($params)
    {
        return $this -> CI -> {$this -> _doc_infos['name'].'_model'} -> set_params_rules($params, $this -> _lang, $this -> _config);
    }

	// --------------------------------------------------------------------

	/**
	 * retourne la liste des catégories des documents
	 *
	 * @access	private
	 * @return	array sous la forme array(array(), array())
	 */
    private function _get_cats()
    {
        $cats = array();

        if (!empty($this -> _writer_rights[$this -> _doc_infos['name'].'_cats_manage'])) {
            $cats = $this -> _get_cats_for_doc();   
        }
        
        return $cats;
    }

	// --------------------------------------------------------------------

	/**
	 * retourne la liste des groupes membres privés
	 *
	 * @access	protected
	 * @return	array 
	 */
    protected function _get_private_members_groups()
    {
        return $this -> CI -> private_groups_helper -> get_all();
    }

	// --------------------------------------------------------------------

	/**
	 * retourne la liste des catégories pour le document
	 *
	 * @access	private
	 * @return	array Tableau contenant les catégories des documents sous la forme array(array(name => name), array(name => name))
	 */
    private function _get_cats_for_doc()
    {
        $cats = array(array(), array());
        $doc_name = $this -> _doc_infos['name'];
        
//        $autorization = (ADMR_IS_ADMIN) ? '1' : $this -> _writer_rights[$doc_name.'_cats_manage'];
        $autorization = $this -> _writer_rights[$doc_name.'_cats_manage'];

        if ($autorization == '1') { // Toutes les catégories
            $this -> CI -> load_helper('categories_helper', $this -> _doc_infos);
            $cats = $this -> CI -> categories_helper -> get_all();         
        }
        elseif ($autorization == '2') { // Uniquement les catégories du rédacteur
            
            $writer_cats = $this -> _writer_rights[$doc_name.'_cats_list_allowed']; // Catégories autorisées pour le rédacteur
           
            foreach($this -> _config['lang'] as $key => $lng) {
                if(!empty($this -> _config['lang'][$key])) {
                    foreach($writer_cats[$key] as $writer_cat) {
                        if (!empty($writer_cat)) {
                            $cats[$key][$writer_cat] = $writer_cat;
                        }              
                    }                   
                }
            }
        }

        return $cats;     
    }

	// --------------------------------------------------------------------

    /**
	 * Vérifie la validité d'accès au document par le rédacteur
     * 
	 * @access public
     * @return void
	 */
    protected function _is_doc_allowed($doc_id)
    {   
        if (!empty($this -> _writer_rights[$this -> _doc_infos['name'].'_manage'])) // Autorisation d'accès à la section du document
        { 
            if (empty($doc_id) && !empty($this -> _writer_rights[$this -> _doc_infos['name'].'_add'])) { // Ajout d'un nouveau document
                return TRUE;
            }
            else {  // Edition d'un document
                $doc = $this -> CI -> writer_docs_model -> get_by_id($doc_id);
               
                if (!empty($doc) && $doc['type'] == $this -> _doc_infos['type'] && !empty($this -> _writer_rights[$this -> _doc_infos['name'].'_edit'])) {
                    return TRUE;
                }   
            }
        }
        
        return FALSE;    
    }

	// --------------------------------------------------------------------

 } // End of class